local lib = LibStub("LibItemBonus-2.0")

if not lib.OnInitialize then return end

local p = lib.patterns

if not p then return end

local function mixin(base, t, force)
	for k, v in pairs(t) do
		if force or not base[k] then
			base[k] = v
		end
	end
end

local function append(base, t)
	for i, entry in ipairs(t) do
		if entry.pattern then
			local found
			for _, v in ipairs(base) do
				if v.pattern == entry.pattern then
					found = true
					break
				end
			end
			if not found then
				table.insert(base, i, entry) -- insert them first
			end
		end
	end
end

local GetEnchantName
do
	local tooltip = lib.tooltip
	tooltip:SetHyperlink"item:38"
	local lines = tooltip:NumLines()
	if lines < 2 then
		GetEnchantName = function () end
	else
		function GetEnchantName(enchantId)
			tooltip:SetHyperlink(string.format("item:38:%d", enchantId))
			local fs = tooltip.lefts[lines + 1]
			if fs and fs:IsShown() then
				return string.gsub(fs:GetText(), "([%%%+%.%-%?%*])", "%%%1")
			end
		end
	end
end

mixin(p.NAMES, {
	STR = SPELL_STAT1_NAME,
	AGI = SPELL_STAT2_NAME,
	STA = SPELL_STAT3_NAME,
	INT = SPELL_STAT4_NAME,
	SPI = SPELL_STAT5_NAME,

	CR_WEAPON = COMBAT_RATING_NAME1,
	CR_DEFENSE = COMBAT_RATING_NAME2,
	CR_DODGE = COMBAT_RATING_NAME3,
	CR_PARRY = COMBAT_RATING_NAME4,
	CR_BLOCK = COMBAT_RATING_NAME5,
	CR_HIT = COMBAT_RATING_NAME6,
	CR_CRIT = COMBAT_RATING_NAME9,
	CR_RESILIENCE = COMBAT_RATING_NAME15,
	CR_EXPERTISE = COMBAT_RATING_NAME24,
})

mixin(p.PATTERNS_GENERIC_LOOKUP, {
	SPELL_STAT1_NAME = "STR",
	SPELL_STAT2_NAME = "AGI",
	SPELL_STAT3_NAME = "STA",
	SPELL_STAT4_NAME = "INT",
	SPELL_STAT5_NAME = "SPI",

	COMBAT_RATING_NAME1 = "CR_WEAPON",
	COMBAT_RATING_NAME2 = "CR_DEFENSE",
	COMBAT_RATING_NAME3 = "CR_DODGE",
	COMBAT_RATING_NAME4 = "CR_PARRY",
	COMBAT_RATING_NAME5 = "CR_BLOCK",
	COMBAT_RATING_NAME6 = "CR_HIT",
	COMBAT_RATING_NAME9 = "CR_CRIT",
	COMBAT_RATING_NAME15 = "CR_RESILIENCE",
	COMBAT_RATING_NAME24 = "CR_EXPERTISE",
})

append(p.PATTERNS_OTHER, {
	{ pattern = EMPTY_SOCKET_BLUE, effect = "EMPTY_SOCKET_BLUE", value = 1},
	{ pattern = EMPTY_SOCKET_META, effect = "EMPTY_SOCKET_META", value = 1},
	{ pattern = EMPTY_SOCKET_RED, effect = "EMPTY_SOCKET_RED", value = 1},
	{ pattern = EMPTY_SOCKET_YELLOW, effect = "EMPTY_SOCKET_YELLOW", value = 1},

	{ pattern = (GetSpellInfo(25117)), effect = "SPELLPOWER", value = 8 }, -- Minor Wizard Oil
	{ pattern = (GetSpellInfo(25119)), effect = "SPELLPOWER", value = 16 }, -- Lesser Wizard Oil
	{ pattern = (GetSpellInfo(25121)), effect = "SPELLPOWER", value = 24 }, -- Wizard Oil
	{ pattern = (GetSpellInfo(25122)), effect = {"SPELLPOWER", "CR_CRIT"}, value = {36, 14} }, -- Brilliant Wizard Oil
	{ pattern = (GetSpellInfo(28017)), effect = "SPELLPOWER", value = 42 }, -- Superior Wizard Oil
	{ pattern = (GetSpellInfo(47906)), effect = "SPELLPOWER", value = 56 }, -- Exceptional Wizard Oil

	{ pattern = (GetSpellInfo(25118)), effect = "MANAREG", value = 4 }, -- Minor Mana Oil
	{ pattern = (GetSpellInfo(25120)), effect = "MANAREG", value = 8 }, -- Lesser Mana Oil
	{ pattern = (GetSpellInfo(25123)), effect = {"MANAREG", "SPELLPOWER"}, value = {12, 13} }, -- "Brilliant Mana Oil"
	{ pattern = (GetSpellInfo(28013)), effect = "MANAREG", value = 14 }, -- Superior Mana Oil
	{ pattern = (GetSpellInfo(47904)), effect = "MANAREG", value = 19 }, -- Exceptional Mana Oil

	{ pattern = (GetSpellInfo(54446)), effect = {"PARRY", "DISARMREDUCTION"}, value = {2, 50} }, -- Rune of Swordbreaking
	{ pattern = (GetSpellInfo(53323)), effect = {"PARRY", "DISARMREDUCTION"}, value = {4, 50} }, -- Rune of Swordshattering
	{ pattern = (GetSpellInfo(62158)), effect = {"DEFENSE", "MOD_STA"}, value = {25, 2} }, -- Rune of the Stoneskin Gargoyle

	-- The following have a different enchant string than their spellid, so require some tooltip scanning
	{ pattern = (GetEnchantName(2603)), effect = "FISHING", value = 5 }, -- Eternium Line
	{ pattern = (GetEnchantName(2656)), effect = {"MANAREG", "HEALTHREG"}, value = {4, 4} }, -- Vitality
	{ pattern = (GetEnchantName(3244)), effect = {"MANAREG", "HEALTHREG"}, value = {6, 6} }, -- Greater Vitality
	{ pattern = (GetEnchantName(2672)), effect = {"FROSTSPELLPOWER", "SHADOWSPELLPOWER"}, value = {54, 54} }, -- Soulfrost
	{ pattern = (GetEnchantName(2671)), effect = {"ARCANESPELLPOWER", "FIRESPELLPOWER"}, value = {50, 50} }, -- Sunfire
	{ pattern = (GetEnchantName(2667)), effect = "ATTACKPOWER", value = 70 }, -- Savagery
	-- { pattern = "Increases defense rating by 5, Shadow resistance by 10 and your normal health regeneration by 3%.", effect = {"CR_DEFENSE", "SHADOWRES", "HEALTHREG_P"}, value = {5, 10, 3} },
	{ pattern = (GetEnchantName(2613)), effect = "THREATREDUCTION", value = -2 }, -- +2% Threat
	{ pattern = (GetEnchantName(2621)), effect = "THREATREDUCTION", value = 2 }, -- Subtlety
	{ pattern = (GetEnchantName(3238)), effect = {"HERBALISM", "MINING", "SKINNING"}, value = {5, 5, 5}, }, -- Gatherer
	{ pattern = (GetEnchantName(3826)), effect = {"CR_CRIT", "CR_HIT"}, value = {12, 12}, }, -- Icewalker
	{ pattern = (GetEnchantName(3247)), effect = "ATTACKPOWERUNDEAD", value = 140, }, -- Scourgebane
	{ pattern = (GetEnchantName(3788)), effect = {"CR_CRIT", "CR_HIT"}, value = {25, 25}, }, -- Accuracy
	{ pattern = (GetEnchantName(3296)), effect = {"THREATREDUCTION", "SPI"}, value = {2, 10}, }, -- Wisddom
	{ pattern = (GetEnchantName(3232)), effect = {"RUNSPEED", "STA"}, value = {8, 15}, }, -- Tuskar's Vitality
	{ pattern = (GetEnchantName(3818)), effect = {"STA", "CR_DEFENSE"}, value = {37, 20}, }, -- Arcanum of the Stalwart Protector
})

